#include "astmanproxy.h"

/* This routine based on get_input from Asterisk manager.c */
/* Good generic line-based input routine for \r\n\r\n terminated input */
/* Used by standard.c and other input handlers */
int get_input(struct mansession *s, char *output)
{
        /* output must have at least sizeof(s->inbuf) space */
        int res;
        int x;
        struct pollfd fds[1];
        char iabuf[INET_ADDRSTRLEN];

        /* Look for \r\n from the front, our preferred end of line */
        for (x=1;x<s->inlen;x++) {
                if ((s->inbuf[x] == '\n') && (s->inbuf[x-1] == '\r')) {
                        /* Copy output data not including \r\n */
                        memcpy(output, s->inbuf, x - 1);
                        /* Add trailing \0 */
                        output[x-1] = '\0';
                        /* Move remaining data back to the front */
                        memmove(s->inbuf, s->inbuf + x + 1, s->inlen - x);
                        s->inlen -= (x + 1);
                        return 1;
                }
        }

        /* TODO: Do we need to look for \n alone or some other variation to fix broken things? */

        if (s->inlen >= sizeof(s->inbuf) - 1) {
            if (debug)
                debugmsg("Warning: Got long line with no end from %s: %s\n", ast_inet_ntoa(iabuf, sizeof(iabuf), s->sin.sin_addr), s->inbuf);
            s->inlen = 0;
        }
        fds[0].fd = s->fd;
        fds[0].events = POLLIN;
        res = poll(fds, 1, -1);
        if (res < 0 && debug) {
                debugmsg("Select returned error");
        } else if (res > 0) {
                pthread_mutex_lock(&s->lock);
                res = read(s->fd, s->inbuf + s->inlen, sizeof(s->inbuf) - 1 - s->inlen);
                pthread_mutex_unlock(&s->lock);
                if (res < 1)
                        return -1;
        }
        s->inlen += res;
        s->inbuf[s->inlen] = '\0';
        return 0;
        /* We have some input, but it's not ready for processing */
}

char *astman_get_header(struct message *m, char *var)
{
        char cmp[80];
        int x;
        snprintf(cmp, sizeof(cmp), "%s: ", var);
        for (x=0;x<m->hdrcount;x++)
                if (!strncasecmp(cmp, m->headers[x], strlen(cmp)))
                        return m->headers[x] + strlen(cmp);
        return "";
}

int AddHeader(struct message *m, const char *fmt, ...) {
    va_list ap;

    int res;

    if (m->hdrcount < MAX_HEADERS - 1) {
        va_start(ap, fmt);
        vsprintf(m->headers[m->hdrcount], fmt, ap);
        va_end(ap);
        m->hdrcount++;
        res = 0;
    } else
        res = 1;

    return res;
}

/* Recursive thread safe replacement of inet_ntoa */
const char *ast_inet_ntoa(char *buf, int bufsiz, struct in_addr ia)
{
        return inet_ntop(AF_INET, &ia, buf, bufsiz);
}

